# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""
from __future__ import annotations

from enum import Enum
from json import loads
from typing import Self


class LookingSimilarModel(str, Enum):
    """
    Looking similar model.  This model recommends items that look similar to the item with the ID `objectID` based on image attributes in your index.
    """

    """
    allowed enum values
    """
    LOOKING_MINUS_SIMILAR = "looking-similar"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of LookingSimilarModel from a JSON string"""
        return cls(loads(json_str))



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""
from __future__ import annotations

from enum import Enum
from json import loads
from typing import Self


class Anchoring(str, Enum):
    """
    Which part of the search query the pattern should match:  - `startsWith`. The pattern must match the begginning of the query. - `endsWith`. The pattern must match the end of the query. - `is`. The pattern must match the query exactly. - `contains`. The pattern must match anywhere in the query.  Empty queries are only allowed as pattern with `anchoring: is`.
    """

    """
    allowed enum values
    """
    IS = "is"
    STARTSWITH = "startsWith"
    ENDSWITH = "endsWith"
    CONTAINS = "contains"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Anchoring from a JSON string"""
        return cls(loads(json_str))



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""
from __future__ import annotations

from json import loads
from typing import Annotated, Any, Dict, Optional, Self

from pydantic import BaseModel, Field, StrictBool, StrictInt

from algoliasearch.recommend.models.matched_geo_location import MatchedGeoLocation
from algoliasearch.recommend.models.personalization import Personalization


class RankingInfo(BaseModel):
    """
    Object with detailed information about the record's ranking.
    """

    filters: Annotated[int, Field(strict=True, ge=0)] = Field(
        description="Whether a filter matched the query."
    )
    first_matched_word: Annotated[int, Field(strict=True, ge=0)] = Field(
        description="Position of the first matched word in the best matching attribute of the record.",
        alias="firstMatchedWord",
    )
    geo_distance: Annotated[int, Field(strict=True, ge=0)] = Field(
        description="Distance between the geo location in the search query and the best matching geo location in the record, divided by the geo precision (in meters).",
        alias="geoDistance",
    )
    geo_precision: Optional[Annotated[int, Field(strict=True, ge=1)]] = Field(
        default=None,
        description="Precision used when computing the geo distance, in meters.",
        alias="geoPrecision",
    )
    matched_geo_location: Optional[MatchedGeoLocation] = Field(
        default=None, alias="matchedGeoLocation"
    )
    personalization: Optional[Personalization] = None
    nb_exact_words: Annotated[int, Field(strict=True, ge=0)] = Field(
        description="Number of exactly matched words.", alias="nbExactWords"
    )
    nb_typos: Annotated[int, Field(strict=True, ge=0)] = Field(
        description="Number of typos encountered when matching the record.",
        alias="nbTypos",
    )
    promoted: StrictBool = Field(
        description="Whether the record was promoted by a rule."
    )
    proximity_distance: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(
        default=None,
        description="Number of words between multiple matches in the query plus 1. For single word queries, `proximityDistance` is 0.",
        alias="proximityDistance",
    )
    user_score: StrictInt = Field(
        description="Overall ranking of the record, expressed as a single integer. This attribute is internal.",
        alias="userScore",
    )
    words: Annotated[int, Field(strict=True, ge=1)] = Field(
        description="Number of matched words."
    )
    promoted_by_re_ranking: Optional[StrictBool] = Field(
        default=None,
        description="Whether the record is re-ranked.",
        alias="promotedByReRanking",
    )

    model_config = {"populate_by_name": True, "validate_assignment": True}

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of RankingInfo from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        if self.matched_geo_location:
            _dict["matchedGeoLocation"] = self.matched_geo_location.to_dict()
        if self.personalization:
            _dict["personalization"] = self.personalization.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of RankingInfo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "filters": obj.get("filters"),
                "firstMatchedWord": obj.get("firstMatchedWord"),
                "geoDistance": obj.get("geoDistance"),
                "geoPrecision": obj.get("geoPrecision"),
                "matchedGeoLocation": (
                    MatchedGeoLocation.from_dict(obj.get("matchedGeoLocation"))
                    if obj.get("matchedGeoLocation") is not None
                    else None
                ),
                "personalization": (
                    Personalization.from_dict(obj.get("personalization"))
                    if obj.get("personalization") is not None
                    else None
                ),
                "nbExactWords": obj.get("nbExactWords"),
                "nbTypos": obj.get("nbTypos"),
                "promoted": obj.get("promoted"),
                "proximityDistance": obj.get("proximityDistance"),
                "userScore": obj.get("userScore"),
                "words": obj.get("words"),
                "promotedByReRanking": obj.get("promotedByReRanking"),
            }
        )
        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""
from __future__ import annotations

from json import loads
from typing import Any, Dict, Optional, Self

from pydantic import BaseModel, Field, StrictBool


class SourceUpdateShopify(BaseModel):
    """
    SourceUpdateShopify
    """

    collection_id_indexing: Optional[StrictBool] = Field(
        default=False,
        description="Whether to index collection IDs.   If your store has `has_collection_search_page` set to true, collection IDs will be indexed even if `collectionIDIndexing` is false.  ",
        alias="collectionIDIndexing",
    )
    increase_product_collection_limit: Optional[StrictBool] = Field(
        default=False,
        description="Whether to increase the number of indexed collections per product. If true, Algolia indexes 200 collections per product. If false, 100 collections per product are indexed. ",
        alias="increaseProductCollectionLimit",
    )
    default_price_ratio_as_one: Optional[StrictBool] = Field(
        default=True,
        description="Whether to set the default price ratio to 1 if no sale price is present.  The price ratio is determined by the ratio: `sale_price` / `regular_price`. If no sale price is present, the price ratio would be 0. If `defaultPriceRatioAsOne` is true, the price ratio is indexed as 1 instead. ",
        alias="defaultPriceRatioAsOne",
    )
    exclude_oos_variants_for_price_at_trs: Optional[StrictBool] = Field(
        default=True,
        description="Whether to exclude out-of-stock variants when determining the `max_variant_price` and `min_variant_price` attributes. ",
        alias="excludeOOSVariantsForPriceAtTRS",
    )
    include_variants_inventory: Optional[StrictBool] = Field(
        default=True,
        description="Whether to include an inventory with every variant for every product record. ",
        alias="includeVariantsInventory",
    )
    has_collection_search_page: Optional[StrictBool] = Field(
        default=False,
        description="Whether to include collection IDs and handles in the product records. ",
        alias="hasCollectionSearchPage",
    )
    product_named_tags: Optional[StrictBool] = Field(
        default=False,
        description="Whether to convert tags on products to named tags.  To learn more, see [Named tags](https://www.algolia.com/doc/integration/shopify/sending-and-managing-data/named-tags). ",
        alias="productNamedTags",
    )

    model_config = {"populate_by_name": True, "validate_assignment": True}

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SourceUpdateShopify from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of SourceUpdateShopify from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "collectionIDIndexing": obj.get("collectionIDIndexing"),
                "increaseProductCollectionLimit": obj.get(
                    "increaseProductCollectionLimit"
                ),
                "defaultPriceRatioAsOne": obj.get("defaultPriceRatioAsOne"),
                "excludeOOSVariantsForPriceAtTRS": obj.get(
                    "excludeOOSVariantsForPriceAtTRS"
                ),
                "includeVariantsInventory": obj.get("includeVariantsInventory"),
                "hasCollectionSearchPage": obj.get("hasCollectionSearchPage"),
                "productNamedTags": obj.get("productNamedTags"),
            }
        )
        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""
from __future__ import annotations

from json import loads
from re import match
from typing import Annotated, Any, Dict, List, Optional, Self

from pydantic import BaseModel, Field, StrictInt, StrictStr, field_validator

from algoliasearch.insights.models.click_event import ClickEvent


class ClickedObjectIDsAfterSearch(BaseModel):
    """
    Click event after an Algolia request.  Use this event to track when users click items in the search results. If you're building your category pages with Algolia, you'll also use this event.
    """

    event_name: Annotated[str, Field(min_length=1, strict=True, max_length=64)] = Field(
        description="Event name, up to 64 ASCII characters.  Consider naming events consistently—for example, by adopting Segment's [object-action](https://segment.com/academy/collecting-data/naming-conventions-for-clean-data/#the-object-action-framework) framework. ",
        alias="eventName",
    )
    event_type: ClickEvent = Field(alias="eventType")
    index: StrictStr = Field(
        description="Index name (case-sensitive) to which the event's items belong."
    )
    object_ids: Annotated[List[StrictStr], Field(min_length=1, max_length=20)] = Field(
        description="Object IDs of the records that are part of the event.",
        alias="objectIDs",
    )
    positions: Annotated[List[StrictInt], Field(min_length=1, max_length=20)] = Field(
        description="Position of the clicked item the search results.  You must provide 1 `position` for each `objectID`. "
    )
    query_id: Annotated[str, Field(min_length=32, strict=True, max_length=32)] = Field(
        description="Unique identifier for a search query.  The query ID is required for events related to search or browse requests. If you add `clickAnalytics: true` as a search request parameter, the query ID is included in the API response. ",
        alias="queryID",
    )
    user_token: Annotated[str, Field(min_length=1, strict=True, max_length=129)] = (
        Field(
            description="Anonymous or pseudonymous user identifier.  Don't use personally identifiable information in user tokens. For more information, see [User token](https://www.algolia.com/doc/guides/sending-events/concepts/usertoken/). ",
            alias="userToken",
        )
    )
    authenticated_user_token: Optional[
        Annotated[str, Field(min_length=1, strict=True, max_length=129)]
    ] = Field(
        default=None,
        description="Identifier for authenticated users.  When the user signs in, you can get an identifier from your system and send it as `authenticatedUserToken`. This lets you keep using the `userToken` from before the user signed in, while providing a reliable way to identify users across sessions. Don't use personally identifiable information in user tokens. For more information, see [User token](https://www.algolia.com/doc/guides/sending-events/concepts/usertoken/). ",
        alias="authenticatedUserToken",
    )
    timestamp: Optional[StrictInt] = Field(
        default=None,
        description="Timestamp of the event, measured in milliseconds since the Unix epoch. By default, the Insights API uses the time it receives an event as its timestamp. ",
    )

    @field_validator("event_name")
    def event_name_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not match(r"[\x20-\x7E]{1,64}", value):
            raise ValueError(
                r"must validate the regular expression /[\x20-\x7E]{1,64}/"
            )
        return value

    @field_validator("query_id")
    def query_id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not match(r"[0-9a-f]{32}", value):
            raise ValueError(r"must validate the regular expression /[0-9a-f]{32}/")
        return value

    @field_validator("user_token")
    def user_token_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not match(r"[a-zA-Z0-9_=\/+-]{1,129}", value):
            raise ValueError(
                r"must validate the regular expression /[a-zA-Z0-9_=\/+-]{1,129}/"
            )
        return value

    @field_validator("authenticated_user_token")
    def authenticated_user_token_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not match(r"[a-zA-Z0-9_=\/+-]{1,129}", value):
            raise ValueError(
                r"must validate the regular expression /[a-zA-Z0-9_=\/+-]{1,129}/"
            )
        return value

    model_config = {"populate_by_name": True, "validate_assignment": True}

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ClickedObjectIDsAfterSearch from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of ClickedObjectIDsAfterSearch from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "eventName": obj.get("eventName"),
                "eventType": obj.get("eventType"),
                "index": obj.get("index"),
                "objectIDs": obj.get("objectIDs"),
                "positions": obj.get("positions"),
                "queryID": obj.get("queryID"),
                "userToken": obj.get("userToken"),
                "authenticatedUserToken": obj.get("authenticatedUserToken"),
                "timestamp": obj.get("timestamp"),
            }
        )
        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""
from __future__ import annotations

from json import dumps
from typing import Annotated, Any, Dict, List, Optional, Self, Tuple, Union
from urllib.parse import quote

from pydantic import Field, StrictStr

from algoliasearch.http.api_response import ApiResponse
from algoliasearch.http.request_options import RequestOptions
from algoliasearch.http.serializer import bodySerializer
from algoliasearch.http.transporter import Transporter
from algoliasearch.http.verb import Verb
from algoliasearch.personalization.config import PersonalizationConfig
from algoliasearch.personalization.models.delete_user_profile_response import (
    DeleteUserProfileResponse,
)
from algoliasearch.personalization.models.get_user_token_response import (
    GetUserTokenResponse,
)
from algoliasearch.personalization.models.personalization_strategy_params import (
    PersonalizationStrategyParams,
)
from algoliasearch.personalization.models.set_personalization_strategy_response import (
    SetPersonalizationStrategyResponse,
)


class PersonalizationClient:
    """The Algolia 'PersonalizationClient' class.

    Args:
    app_id (str): The Algolia application ID to retrieve information from.
    api_key (str): The Algolia api key bound to the given `app_id`.
    region ("eu" | "us"): The region of your Algolia application.

    Returns:
    The initialized API client.

    Example:
    _client = PersonalizationClient("YOUR_ALGOLIA_APP_ID", "YOUR_ALGOLIA_API_KEY", region="'eu' or 'us'")
    _client_with_named_args = PersonalizationClient(app_id="YOUR_ALGOLIA_APP_ID", api_key="YOUR_ALGOLIA_API_KEY", region="'eu' or 'us'")

    See `PersonalizationClient.create_with_config` for advanced configuration.
    """

    _transporter: Transporter
    _config: PersonalizationConfig
    _request_options: RequestOptions

    def __init__(
        self,
        app_id: Optional[str] = None,
        api_key: Optional[str] = None,
        region: str = None,
        transporter: Optional[Transporter] = None,
        config: Optional[PersonalizationConfig] = None,
    ) -> None:
        if transporter is not None and config is None:
            config = transporter._config

        if config is None:
            config = PersonalizationConfig(app_id, api_key, region)
        self._config = config
        self._request_options = RequestOptions(config)

        if transporter is None:
            transporter = Transporter(config)
        self._transporter = transporter

    def create_with_config(
        config: PersonalizationConfig, transporter: Optional[Transporter] = None
    ) -> Self:
        """Allows creating a client with a customized `PersonalizationConfig` and `Transporter`. If `transporter` is not provided, the default one will be initialized from the given `config`.

        Args:
        config (PersonalizationConfig): The config of the API client.
        transporter (Transporter): The HTTP transporter, see `http/transporter.py` for implementation details.

        Returns:
        The initialized API client.

        Example:
        _client_with_custom_config = PersonalizationClient.create_with_config(config=PersonalizationConfig(...))
        _client_with_custom_config_and_transporter = PersonalizationClient.create_with_config(config=PersonalizationConfig(...), transporter=Transporter(...))
        """
        if transporter is None:
            transporter = Transporter(config)

        return PersonalizationClient(
            app_id=config.app_id,
            api_key=config.api_key,
            region=config.region,
            transporter=transporter,
            config=config,
        )

    async def __aenter__(self) -> None:
        return self

    async def __aexit__(self, exc_type, exc_value, traceback) -> None:
        """Closes the underlying `transporter` of the API client."""
        await self.close()

    async def close(self) -> None:
        """Closes the underlying `transporter` of the API client."""
        return await self._transporter.close()

    async def custom_delete_with_http_info(
        self,
        path: Annotated[
            StrictStr,
            Field(
                description='Path of the endpoint, anything after "/1" must be specified.'
            ),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        This method allow you to send requests to the Algolia REST API.


        :param path: Path of the endpoint, anything after \"/1\" must be specified. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if path is None:
            raise ValueError(
                "Parameter `path` is required when calling `custom_delete`."
            )

        _query_parameters: List[Tuple[str, str]] = []

        if parameters is not None:
            for _qpkey, _qpvalue in parameters.items():
                _query_parameters.append((_qpkey, _qpvalue))

        return await self._transporter.request(
            verb=Verb.DELETE,
            path="/{path}".replace("{path}", path),
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def custom_delete(
        self,
        path: Annotated[
            StrictStr,
            Field(
                description='Path of the endpoint, anything after "/1" must be specified.'
            ),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> object:
        """
        This method allow you to send requests to the Algolia REST API.


        :param path: Path of the endpoint, anything after \"/1\" must be specified. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'object' result object.
        """
        return (
            await self.custom_delete_with_http_info(path, parameters, request_options)
        ).deserialize(object)

    async def custom_get_with_http_info(
        self,
        path: Annotated[
            StrictStr,
            Field(
                description='Path of the endpoint, anything after "/1" must be specified.'
            ),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        This method allow you to send requests to the Algolia REST API.


        :param path: Path of the endpoint, anything after \"/1\" must be specified. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if path is None:
            raise ValueError("Parameter `path` is required when calling `custom_get`.")

        _query_parameters: List[Tuple[str, str]] = []

        if parameters is not None:
            for _qpkey, _qpvalue in parameters.items():
                _query_parameters.append((_qpkey, _qpvalue))

        return await self._transporter.request(
            verb=Verb.GET,
            path="/{path}".replace("{path}", path),
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def custom_get(
        self,
        path: Annotated[
            StrictStr,
            Field(
                description='Path of the endpoint, anything after "/1" must be specified.'
            ),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> object:
        """
        This method allow you to send requests to the Algolia REST API.


        :param path: Path of the endpoint, anything after \"/1\" must be specified. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'object' result object.
        """
        return (
            await self.custom_get_with_http_info(path, parameters, request_options)
        ).deserialize(object)

    async def custom_post_with_http_info(
        self,
        path: Annotated[
            StrictStr,
            Field(
                description='Path of the endpoint, anything after "/1" must be specified.'
            ),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        body: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Parameters to send with the custom request."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        This method allow you to send requests to the Algolia REST API.


        :param path: Path of the endpoint, anything after \"/1\" must be specified. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param body: Parameters to send with the custom request.
        :type body: object
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if path is None:
            raise ValueError("Parameter `path` is required when calling `custom_post`.")

        _query_parameters: List[Tuple[str, str]] = []

        if parameters is not None:
            for _qpkey, _qpvalue in parameters.items():
                _query_parameters.append((_qpkey, _qpvalue))

        _data = {}
        if body is not None:
            _data = body

        return await self._transporter.request(
            verb=Verb.POST,
            path="/{path}".replace("{path}", path),
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                data=dumps(bodySerializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def custom_post(
        self,
        path: Annotated[
            StrictStr,
            Field(
                description='Path of the endpoint, anything after "/1" must be specified.'
            ),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        body: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Parameters to send with the custom request."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> object:
        """
        This method allow you to send requests to the Algolia REST API.


        :param path: Path of the endpoint, anything after \"/1\" must be specified. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param body: Parameters to send with the custom request.
        :type body: object
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'object' result object.
        """
        return (
            await self.custom_post_with_http_info(
                path, parameters, body, request_options
            )
        ).deserialize(object)

    async def custom_put_with_http_info(
        self,
        path: Annotated[
            StrictStr,
            Field(
                description='Path of the endpoint, anything after "/1" must be specified.'
            ),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        body: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Parameters to send with the custom request."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        This method allow you to send requests to the Algolia REST API.


        :param path: Path of the endpoint, anything after \"/1\" must be specified. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param body: Parameters to send with the custom request.
        :type body: object
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if path is None:
            raise ValueError("Parameter `path` is required when calling `custom_put`.")

        _query_parameters: List[Tuple[str, str]] = []

        if parameters is not None:
            for _qpkey, _qpvalue in parameters.items():
                _query_parameters.append((_qpkey, _qpvalue))

        _data = {}
        if body is not None:
            _data = body

        return await self._transporter.request(
            verb=Verb.PUT,
            path="/{path}".replace("{path}", path),
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                data=dumps(bodySerializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def custom_put(
        self,
        path: Annotated[
            StrictStr,
            Field(
                description='Path of the endpoint, anything after "/1" must be specified.'
            ),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        body: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Parameters to send with the custom request."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> object:
        """
        This method allow you to send requests to the Algolia REST API.


        :param path: Path of the endpoint, anything after \"/1\" must be specified. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param body: Parameters to send with the custom request.
        :type body: object
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'object' result object.
        """
        return (
            await self.custom_put_with_http_info(
                path, parameters, body, request_options
            )
        ).deserialize(object)

    async def delete_user_profile_with_http_info(
        self,
        user_token: Annotated[
            StrictStr,
            Field(
                description="Unique identifier representing a user for which to fetch the personalization profile."
            ),
        ],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Deletes a user profile.  The response includes a date and time when the user profile can safely be considered deleted.

        Required API Key ACLs:
          - recommendation

        :param user_token: Unique identifier representing a user for which to fetch the personalization profile. (required)
        :type user_token: str
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if user_token is None:
            raise ValueError(
                "Parameter `user_token` is required when calling `delete_user_profile`."
            )

        return await self._transporter.request(
            verb=Verb.DELETE,
            path="/1/profiles/{userToken}".replace(
                "{userToken}", quote(str(user_token), safe="")
            ),
            request_options=self._request_options.merge(
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def delete_user_profile(
        self,
        user_token: Annotated[
            StrictStr,
            Field(
                description="Unique identifier representing a user for which to fetch the personalization profile."
            ),
        ],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> DeleteUserProfileResponse:
        """
        Deletes a user profile.  The response includes a date and time when the user profile can safely be considered deleted.

        Required API Key ACLs:
          - recommendation

        :param user_token: Unique identifier representing a user for which to fetch the personalization profile. (required)
        :type user_token: str
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'DeleteUserProfileResponse' result object.
        """
        return (
            await self.delete_user_profile_with_http_info(user_token, request_options)
        ).deserialize(DeleteUserProfileResponse)

    async def get_personalization_strategy_with_http_info(
        self, request_options: Optional[Union[dict, RequestOptions]] = None
    ) -> ApiResponse[str]:
        """
        Retrieves the current personalization strategy.

        Required API Key ACLs:
          - recommendation

        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        return await self._transporter.request(
            verb=Verb.GET,
            path="/1/strategies/personalization",
            request_options=self._request_options.merge(
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def get_personalization_strategy(
        self, request_options: Optional[Union[dict, RequestOptions]] = None
    ) -> PersonalizationStrategyParams:
        """
        Retrieves the current personalization strategy.

        Required API Key ACLs:
          - recommendation

        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'PersonalizationStrategyParams' result object.
        """
        return (
            await self.get_personalization_strategy_with_http_info(request_options)
        ).deserialize(PersonalizationStrategyParams)

    async def get_user_token_profile_with_http_info(
        self,
        user_token: Annotated[
            StrictStr,
            Field(
                description="Unique identifier representing a user for which to fetch the personalization profile."
            ),
        ],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Retrieves a user profile and their affinities for different facets.

        Required API Key ACLs:
          - recommendation

        :param user_token: Unique identifier representing a user for which to fetch the personalization profile. (required)
        :type user_token: str
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if user_token is None:
            raise ValueError(
                "Parameter `user_token` is required when calling `get_user_token_profile`."
            )

        return await self._transporter.request(
            verb=Verb.GET,
            path="/1/profiles/personalization/{userToken}".replace(
                "{userToken}", quote(str(user_token), safe="")
            ),
            request_options=self._request_options.merge(
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def get_user_token_profile(
        self,
        user_token: Annotated[
            StrictStr,
            Field(
                description="Unique identifier representing a user for which to fetch the personalization profile."
            ),
        ],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> GetUserTokenResponse:
        """
        Retrieves a user profile and their affinities for different facets.

        Required API Key ACLs:
          - recommendation

        :param user_token: Unique identifier representing a user for which to fetch the personalization profile. (required)
        :type user_token: str
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'GetUserTokenResponse' result object.
        """
        return (
            await self.get_user_token_profile_with_http_info(
                user_token, request_options
            )
        ).deserialize(GetUserTokenResponse)

    async def set_personalization_strategy_with_http_info(
        self,
        personalization_strategy_params: PersonalizationStrategyParams,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Creates a new personalization strategy.

        Required API Key ACLs:
          - recommendation

        :param personalization_strategy_params: (required)
        :type personalization_strategy_params: PersonalizationStrategyParams
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if personalization_strategy_params is None:
            raise ValueError(
                "Parameter `personalization_strategy_params` is required when calling `set_personalization_strategy`."
            )

        _data = {}
        if personalization_strategy_params is not None:
            _data = personalization_strategy_params

        return await self._transporter.request(
            verb=Verb.POST,
            path="/1/strategies/personalization",
            request_options=self._request_options.merge(
                data=dumps(bodySerializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def set_personalization_strategy(
        self,
        personalization_strategy_params: PersonalizationStrategyParams,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> SetPersonalizationStrategyResponse:
        """
        Creates a new personalization strategy.

        Required API Key ACLs:
          - recommendation

        :param personalization_strategy_params: (required)
        :type personalization_strategy_params: PersonalizationStrategyParams
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'SetPersonalizationStrategyResponse' result object.
        """
        return (
            await self.set_personalization_strategy_with_http_info(
                personalization_strategy_params, request_options
            )
        ).deserialize(SetPersonalizationStrategyResponse)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""
from __future__ import annotations

from json import loads
from typing import Annotated, Any, Dict, List, Self

from pydantic import BaseModel, Field, StrictInt, StrictStr

from algoliasearch.search.models.user_hit import UserHit


class SearchUserIdsResponse(BaseModel):
    """
    userIDs data.
    """

    hits: List[UserHit] = Field(description="User objects that match the query.")
    nb_hits: StrictInt = Field(description="Number of results (hits).", alias="nbHits")
    page: Annotated[int, Field(strict=True, ge=0)] = Field(
        description="Page of search results to retrieve."
    )
    hits_per_page: Annotated[int, Field(le=1000, strict=True, ge=1)] = Field(
        description="Maximum number of hits per page.", alias="hitsPerPage"
    )
    updated_at: StrictStr = Field(
        description="Date and time when the object was updated, in RFC 3339 format.",
        alias="updatedAt",
    )

    model_config = {"populate_by_name": True, "validate_assignment": True}

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SearchUserIdsResponse from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        _items = []
        if self.hits:
            for _item in self.hits:
                if _item:
                    _items.append(_item.to_dict())
            _dict["hits"] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of SearchUserIdsResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "hits": (
                    [UserHit.from_dict(_item) for _item in obj.get("hits")]
                    if obj.get("hits") is not None
                    else None
                ),
                "nbHits": obj.get("nbHits"),
                "page": obj.get("page"),
                "hitsPerPage": obj.get("hitsPerPage"),
                "updatedAt": obj.get("updatedAt"),
            }
        )
        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""
from __future__ import annotations

from enum import Enum
from json import loads
from typing import Self


class Statistic(str, Enum):
    """
    Statistic
    """

    """
    allowed enum values
    """
    STAR = "*"
    SEARCH_OPERATIONS = "search_operations"
    TOTAL_SEARCH_OPERATIONS = "total_search_operations"
    TOTAL_SEARCH_REQUESTS = "total_search_requests"
    QUERIES_OPERATIONS = "queries_operations"
    MULTI_QUERIES_OPERATIONS = "multi_queries_operations"
    ACL_OPERATIONS = "acl_operations"
    TOTAL_ACL_OPERATIONS = "total_acl_operations"
    GET_API_KEYS_OPERATIONS = "get_api_keys_operations"
    GET_API_KEY_OPERATIONS = "get_api_key_operations"
    ADD_API_KEY_OPERATIONS = "add_api_key_operations"
    UPDATE_API_KEY_OPERATIONS = "update_api_key_operations"
    DELETE_API_KEY_OPERATIONS = "delete_api_key_operations"
    LIST_API_KEY_OPERATIONS = "list_api_key_operations"
    INDEXING_OPERATIONS = "indexing_operations"
    TOTAL_INDEXING_OPERATIONS = "total_indexing_operations"
    BROWSE_OPERATIONS = "browse_operations"
    CLEAR_INDEX_OPERATIONS = "clear_index_operations"
    COPY_MOVE_OPERATIONS = "copy_move_operations"
    DELETE_INDEX_OPERATIONS = "delete_index_operations"
    GET_LOG_OPERATIONS = "get_log_operations"
    GET_SETTINGS_OPERATIONS = "get_settings_operations"
    SET_SETTINGS_OPERATIONS = "set_settings_operations"
    LIST_INDICES_OPERATIONS = "list_indices_operations"
    WAIT_TASK_OPERATIONS = "wait_task_operations"
    RECORD_OPERATIONS = "record_operations"
    TOTAL_RECORDS_OPERATIONS = "total_records_operations"
    ADD_RECORD_OPERATIONS = "add_record_operations"
    BATCH_OPERATIONS = "batch_operations"
    DELETE_BY_QUERY_OPERATIONS = "delete_by_query_operations"
    DELETE_RECORD_OPERATIONS = "delete_record_operations"
    GET_RECORD_OPERATIONS = "get_record_operations"
    PARTIAL_UPDATE_RECORD_OPERATIONS = "partial_update_record_operations"
    UPDATE_RECORD_OPERATIONS = "update_record_operations"
    SYNONYM_OPERATIONS = "synonym_operations"
    TOTAL_SYNONYM_OPERATIONS = "total_synonym_operations"
    BATCH_SYNONYM_OPERATIONS = "batch_synonym_operations"
    CLEAR_SYNONYM_OPERATIONS = "clear_synonym_operations"
    DELETE_SYNONYM_OPERATIONS = "delete_synonym_operations"
    GET_SYNONYM_OPERATIONS = "get_synonym_operations"
    QUERY_SYNONYM_OPERATIONS = "query_synonym_operations"
    UPDATE_SYNONYM_OPERATIONS = "update_synonym_operations"
    RULE_OPERATIONS = "rule_operations"
    TOTAL_RULES_OPERATIONS = "total_rules_operations"
    BATCH_RULES_OPERATIONS = "batch_rules_operations"
    CLEAR_RULES_OPERATIONS = "clear_rules_operations"
    DELETE_RULES_OPERATIONS = "delete_rules_operations"
    GET_RULES_OPERATIONS = "get_rules_operations"
    SAVE_RULES_OPERATIONS = "save_rules_operations"
    SEARCH_RULES_OPERATIONS = "search_rules_operations"
    TOTAL_RECOMMEND_REQUESTS = "total_recommend_requests"
    TOTAL_WRITE_OPERATIONS = "total_write_operations"
    TOTAL_READ_OPERATIONS = "total_read_operations"
    TOTAL_OPERATIONS = "total_operations"
    QUERYSUGGESTIONS_TOTAL_SEARCH_OPERATIONS = (
        "querysuggestions_total_search_operations"
    )
    QUERYSUGGESTIONS_TOTAL_SEARCH_REQUESTS = "querysuggestions_total_search_requests"
    QUERYSUGGESTIONS_TOTAL_ACL_OPERATIONS = "querysuggestions_total_acl_operations"
    QUERYSUGGESTIONS_TOTAL_INDEXING_OPERATIONS = (
        "querysuggestions_total_indexing_operations"
    )
    QUERYSUGGESTIONS_TOTAL_RECORDS_OPERATIONS = (
        "querysuggestions_total_records_operations"
    )
    QUERYSUGGESTIONS_TOTAL_SYNONYM_OPERATIONS = (
        "querysuggestions_total_synonym_operations"
    )
    QUERYSUGGESTIONS_TOTAL_RULES_OPERATIONS = "querysuggestions_total_rules_operations"
    QUERYSUGGESTIONS_TOTAL_WRITE_OPERATIONS = "querysuggestions_total_write_operations"
    QUERYSUGGESTIONS_TOTAL_READ_OPERATIONS = "querysuggestions_total_read_operations"
    QUERYSUGGESTIONS_TOTAL_OPERATIONS = "querysuggestions_total_operations"
    AVG_PROCESSING_TIME = "avg_processing_time"
    ENUM_90P_PROCESSING_TIME = "90p_processing_time"
    ENUM_99P_PROCESSING_TIME = "99p_processing_time"
    QUERIES_ABOVE_LAST_MS_PROCESSING_TIME = "queries_above_last_ms_processing_time"
    RECORDS = "records"
    DATA_SIZE = "data_size"
    FILE_SIZE = "file_size"
    MAX_QPS = "max_qps"
    REGION_MAX_QPS = "region_max_qps"
    TOTAL_MAX_QPS = "total_max_qps"
    USED_SEARCH_CAPACITY = "used_search_capacity"
    AVG_USED_SEARCH_CAPACITY = "avg_used_search_capacity"
    REGION_USED_SEARCH_CAPACITY = "region_used_search_capacity"
    REGION_AVG_USED_SEARCH_CAPACITY = "region_avg_used_search_capacity"
    TOTAL_USED_SEARCH_CAPACITY = "total_used_search_capacity"
    TOTAL_AVG_USED_SEARCH_CAPACITY = "total_avg_used_search_capacity"
    DEGRADED_QUERIES_SSD_USED_QUERIES_IMPACTED = (
        "degraded_queries_ssd_used_queries_impacted"
    )
    DEGRADED_QUERIES_SSD_USED_SECONDS_IMPACTED = (
        "degraded_queries_ssd_used_seconds_impacted"
    )
    DEGRADED_QUERIES_MAX_CAPACITY_QUERIES_IMPACTED = (
        "degraded_queries_max_capacity_queries_impacted"
    )
    DEGRADED_QUERIES_MAX_CAPACITY_SECONDS_IMPACTED = (
        "degraded_queries_max_capacity_seconds_impacted"
    )

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Statistic from a JSON string"""
        return cls(loads(json_str))



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""
from __future__ import annotations

from json import loads
from typing import Any, Dict, List, Optional, Self

from pydantic import BaseModel, Field, StrictStr

from algoliasearch.search.models.synonym_type import SynonymType


class SynonymHit(BaseModel):
    """
    Synonym object.
    """

    object_id: StrictStr = Field(
        description="Unique identifier of a synonym object.", alias="objectID"
    )
    type: SynonymType
    synonyms: Optional[List[StrictStr]] = Field(
        default=None, description="Words or phrases considered equivalent."
    )
    input: Optional[StrictStr] = Field(
        default=None,
        description="Word or phrase to appear in query strings (for [`onewaysynonym`s](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/adding-synonyms/in-depth/one-way-synonyms/)).",
    )
    word: Optional[StrictStr] = Field(
        default=None,
        description="Word or phrase to appear in query strings (for [`altcorrection1` and `altcorrection2`](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/adding-synonyms/in-depth/synonyms-alternative-corrections/)).",
    )
    corrections: Optional[List[StrictStr]] = Field(
        default=None, description="Words to be matched in records."
    )
    placeholder: Optional[StrictStr] = Field(
        default=None,
        description="[Placeholder token](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/adding-synonyms/in-depth/synonyms-placeholders/) to be put inside records. ",
    )
    replacements: Optional[List[StrictStr]] = Field(
        default=None,
        description="Query words that will match the [placeholder token](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/adding-synonyms/in-depth/synonyms-placeholders/).",
    )

    model_config = {"populate_by_name": True, "validate_assignment": True}

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SynonymHit from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of SynonymHit from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "objectID": obj.get("objectID"),
                "type": obj.get("type"),
                "synonyms": obj.get("synonyms"),
                "input": obj.get("input"),
                "word": obj.get("word"),
                "corrections": obj.get("corrections"),
                "placeholder": obj.get("placeholder"),
                "replacements": obj.get("replacements"),
            }
        )
        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""
from __future__ import annotations

from json import loads
from typing import Annotated, Any, Dict, List, Optional, Self, Union

from pydantic import BaseModel, Field, StrictInt, StrictStr

from algoliasearch.analytics.models.click_positions_inner import ClickPositionsInner


class TopSearchWithAnalytics(BaseModel):
    """
    TopSearchWithAnalytics
    """

    search: StrictStr = Field(description="Search query.")
    count: StrictInt = Field(description="Number of searches.")
    click_through_rate: Optional[
        Union[
            Annotated[float, Field(le=1, strict=True, ge=0)],
            Annotated[int, Field(le=1, strict=True, ge=0)],
        ]
    ] = Field(
        description="Click-through rate, calculated as number of tracked searches with at least one click event divided by the number of tracked searches. If null, Algolia didn't receive any search requests with `clickAnalytics` set to true. ",
        alias="clickThroughRate",
    )
    average_click_position: Optional[
        Union[
            Annotated[float, Field(strict=True, ge=1)],
            Annotated[int, Field(strict=True, ge=1)],
        ]
    ] = Field(
        description="Average position of a clicked search result in the list of search results. If null, Algolia didn't receive any search requests with `clickAnalytics` set to true. ",
        alias="averageClickPosition",
    )
    click_positions: Annotated[
        List[ClickPositionsInner], Field(min_length=12, max_length=12)
    ] = Field(
        description="List of positions in the search results and clicks associated with this search.",
        alias="clickPositions",
    )
    conversion_rate: Optional[
        Union[
            Annotated[float, Field(le=1, strict=True, ge=0)],
            Annotated[int, Field(le=1, strict=True, ge=0)],
        ]
    ] = Field(
        description="Conversion rate, calculated as number of tracked searches with at least one conversion event divided by the number of tracked searches. If null, Algolia didn't receive any search requests with `clickAnalytics` set to true. ",
        alias="conversionRate",
    )
    tracked_search_count: StrictInt = Field(
        description="Number of tracked searches. Tracked searches are search requests where the `clickAnalytics` parameter is true.",
        alias="trackedSearchCount",
    )
    click_count: Annotated[int, Field(strict=True, ge=0)] = Field(
        description="Number of clicks associated with this search.", alias="clickCount"
    )
    conversion_count: Annotated[int, Field(strict=True, ge=0)] = Field(
        description="Number of conversions from this search.", alias="conversionCount"
    )
    nb_hits: StrictInt = Field(description="Number of results (hits).", alias="nbHits")

    model_config = {"populate_by_name": True, "validate_assignment": True}

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of TopSearchWithAnalytics from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        _items = []
        if self.click_positions:
            for _item in self.click_positions:
                if _item:
                    _items.append(_item.to_dict())
            _dict["clickPositions"] = _items
        # set to None if click_through_rate (nullable) is None
        # and model_fields_set contains the field
        if (
            self.click_through_rate is None
            and "click_through_rate" in self.model_fields_set
        ):
            _dict["clickThroughRate"] = None

        # set to None if average_click_position (nullable) is None
        # and model_fields_set contains the field
        if (
            self.average_click_position is None
            and "average_click_position" in self.model_fields_set
        ):
            _dict["averageClickPosition"] = None

        # set to None if conversion_rate (nullable) is None
        # and model_fields_set contains the field
        if self.conversion_rate is None and "conversion_rate" in self.model_fields_set:
            _dict["conversionRate"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of TopSearchWithAnalytics from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "search": obj.get("search"),
                "count": obj.get("count"),
                "clickThroughRate": obj.get("clickThroughRate"),
                "averageClickPosition": obj.get("averageClickPosition"),
                "clickPositions": (
                    [
                        ClickPositionsInner.from_dict(_item)
                        for _item in obj.get("clickPositions")
                    ]
                    if obj.get("clickPositions") is not None
                    else None
                ),
                "conversionRate": obj.get("conversionRate"),
                "trackedSearchCount": obj.get("trackedSearchCount"),
                "clickCount": obj.get("clickCount"),
                "conversionCount": obj.get("conversionCount"),
                "nbHits": obj.get("nbHits"),
            }
        )
        return _obj



# coding: utf-8

from asyncio import sleep
from typing import Callable, Protocol, TypeVar

T = TypeVar("T")


class Timeout(Protocol):
    def __call__(self) -> int:
        return 0

    def __init__(self) -> None:
        pass


class RetryTimeout(Timeout):
    def __call__(self, retry_count: int) -> int:
        return min(retry_count * 0.2, 5)

    def __init__(self) -> None:
        pass


async def create_iterable(
    func: Callable[[T], T],
    validate: Callable[[T], bool],
    aggregator: Callable[[T], None],
    timeout: Timeout = Timeout(),
    error_validate: Callable[[T], bool] = None,
    error_message: Callable[[T], str] = None,
) -> T:
    """
    Helper: Iterates until the given `func` until `timeout` or `validate`.
    """

    async def retry(prev: T = None) -> T:
        resp = await func(prev)

        if aggregator:
            aggregator(resp)

        if validate(resp):
            return resp

        if error_validate is not None and error_validate(resp):
            if error_message is None:
                raise Exception("An error occurred")
            raise Exception(error_message(resp))

        await sleep(timeout())
        return await retry(resp)

    return await retry()



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""
from __future__ import annotations

from json import loads
from typing import Any, Dict, List, Optional, Self, Union

from pydantic import BaseModel, Field, StrictFloat, StrictInt, StrictStr

from algoliasearch.abtesting.models.ab_test_configuration import ABTestConfiguration
from algoliasearch.abtesting.models.status import Status
from algoliasearch.abtesting.models.variant import Variant


class ABTest(BaseModel):
    """
    ABTest
    """

    ab_test_id: StrictInt = Field(
        description="Unique A/B test identifier.", alias="abTestID"
    )
    click_significance: Optional[Union[StrictFloat, StrictInt]] = Field(
        alias="clickSignificance"
    )
    conversion_significance: Optional[Union[StrictFloat, StrictInt]] = Field(
        alias="conversionSignificance"
    )
    add_to_cart_significance: Optional[Union[StrictFloat, StrictInt]] = Field(
        alias="addToCartSignificance"
    )
    purchase_significance: Optional[Union[StrictFloat, StrictInt]] = Field(
        alias="purchaseSignificance"
    )
    revenue_significance: Optional[Dict[str, Union[StrictFloat, StrictInt]]] = Field(
        alias="revenueSignificance"
    )
    updated_at: StrictStr = Field(
        description="Date and time when the A/B test was last updated, in RFC 3339 format.",
        alias="updatedAt",
    )
    created_at: StrictStr = Field(
        description="Date and time when the A/B test was created, in RFC 3339 format.",
        alias="createdAt",
    )
    end_at: StrictStr = Field(
        description="End date and time of the A/B test, in RFC 3339 format.",
        alias="endAt",
    )
    name: StrictStr = Field(description="A/B test name.")
    status: Status
    variants: List[Variant] = Field(
        description="A/B test variants.  The first variant is your _control_ index, typically your production index. The second variant is an index with changed settings that you want to test against the control. "
    )
    configuration: Optional[ABTestConfiguration] = None

    model_config = {"populate_by_name": True, "validate_assignment": True}

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ABTest from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        _items = []
        if self.variants:
            for _item in self.variants:
                if _item:
                    _items.append(_item.to_dict())
            _dict["variants"] = _items
        if self.configuration:
            _dict["configuration"] = self.configuration.to_dict()
        # set to None if click_significance (nullable) is None
        # and model_fields_set contains the field
        if (
            self.click_significance is None
            and "click_significance" in self.model_fields_set
        ):
            _dict["clickSignificance"] = None

        # set to None if conversion_significance (nullable) is None
        # and model_fields_set contains the field
        if (
            self.conversion_significance is None
            and "conversion_significance" in self.model_fields_set
        ):
            _dict["conversionSignificance"] = None

        # set to None if add_to_cart_significance (nullable) is None
        # and model_fields_set contains the field
        if (
            self.add_to_cart_significance is None
            and "add_to_cart_significance" in self.model_fields_set
        ):
            _dict["addToCartSignificance"] = None

        # set to None if purchase_significance (nullable) is None
        # and model_fields_set contains the field
        if (
            self.purchase_significance is None
            and "purchase_significance" in self.model_fields_set
        ):
            _dict["purchaseSignificance"] = None

        # set to None if revenue_significance (nullable) is None
        # and model_fields_set contains the field
        if (
            self.revenue_significance is None
            and "revenue_significance" in self.model_fields_set
        ):
            _dict["revenueSignificance"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of ABTest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "abTestID": obj.get("abTestID"),
                "clickSignificance": obj.get("clickSignificance"),
                "conversionSignificance": obj.get("conversionSignificance"),
                "addToCartSignificance": obj.get("addToCartSignificance"),
                "purchaseSignificance": obj.get("purchaseSignificance"),
                "revenueSignificance": obj.get("revenueSignificance"),
                "updatedAt": obj.get("updatedAt"),
                "createdAt": obj.get("createdAt"),
                "endAt": obj.get("endAt"),
                "name": obj.get("name"),
                "status": obj.get("status"),
                "variants": (
                    [Variant.from_dict(_item) for _item in obj.get("variants")]
                    if obj.get("variants") is not None
                    else None
                ),
                "configuration": (
                    ABTestConfiguration.from_dict(obj.get("configuration"))
                    if obj.get("configuration") is not None
                    else None
                ),
            }
        )
        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""
from __future__ import annotations

from json import loads
from typing import Any, Dict, Optional, Self

from pydantic import BaseModel

from algoliasearch.search.models.dictionary_language import DictionaryLanguage


class Languages(BaseModel):
    """
    Dictionary language.
    """

    plurals: Optional[DictionaryLanguage]
    stopwords: Optional[DictionaryLanguage]
    compounds: Optional[DictionaryLanguage]

    model_config = {"populate_by_name": True, "validate_assignment": True}

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Languages from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        if self.plurals:
            _dict["plurals"] = self.plurals.to_dict()
        if self.stopwords:
            _dict["stopwords"] = self.stopwords.to_dict()
        if self.compounds:
            _dict["compounds"] = self.compounds.to_dict()
        # set to None if plurals (nullable) is None
        # and model_fields_set contains the field
        if self.plurals is None and "plurals" in self.model_fields_set:
            _dict["plurals"] = None

        # set to None if stopwords (nullable) is None
        # and model_fields_set contains the field
        if self.stopwords is None and "stopwords" in self.model_fields_set:
            _dict["stopwords"] = None

        # set to None if compounds (nullable) is None
        # and model_fields_set contains the field
        if self.compounds is None and "compounds" in self.model_fields_set:
            _dict["compounds"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of Languages from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "plurals": (
                    DictionaryLanguage.from_dict(obj.get("plurals"))
                    if obj.get("plurals") is not None
                    else None
                ),
                "stopwords": (
                    DictionaryLanguage.from_dict(obj.get("stopwords"))
                    if obj.get("stopwords") is not None
                    else None
                ),
                "compounds": (
                    DictionaryLanguage.from_dict(obj.get("compounds"))
                    if obj.get("compounds") is not None
                    else None
                ),
            }
        )
        return _obj



from os import environ
from typing import Dict, Optional

from algoliasearch.http.hosts import HostsCollection


class BaseConfig:
    app_id: Optional[str]
    api_key: Optional[str]

    read_timeout: int
    write_timeout: int
    connect_timeout: int

    wait_task_time_before_retry: Optional[int]

    headers: Dict[str, str]
    proxies: Dict[str, str]

    hosts: HostsCollection

    def __init__(self, app_id: Optional[str] = None, api_key: Optional[str] = None):
        app_id = environ.get("ALGOLIA_APP_ID") if app_id is None else app_id

        assert app_id, "`app_id` is missing."

        api_key = environ.get("ALGOLIA_API_KEY") if api_key is None else api_key

        assert api_key, "`api_key` is missing."

        self.app_id = str(app_id)
        self.api_key = str(api_key)

        # In milliseconds
        self.read_timeout = 5000
        self.write_timeout = 30000
        self.connect_timeout = 2000

        self.wait_task_time_before_retry = None
        self.headers = None
        self.proxies = None
        self.hosts = None



from platform import python_version
from typing import Optional, Self

from algoliasearch import __version__


class UserAgent:
    def get(self) -> str:
        return self.value

    def add(self, segment: str, version: Optional[str] = __version__) -> Self:
        self.value += "; {} ({})".format(segment, version)
        return self

    def __init__(self) -> None:
        self.value = "Algolia for Python ({}); Python ({})".format(
            __version__, str(python_version())
        )
